unit ESBColoredNumbers;

interface

uses
	Messages, WinProcs, WinTypes, Controls, Classes, SysUtils, Graphics,
	RVDefine, RVUtil, RVClass, RPDefine, RPBase, RVCsData, RVProj;

type
	TESBColNumDataText = class (TRaveDataText)
	protected
		fLessColor: TColor;
		fGreaterColor: TColor;
		fEqualColor: TColor;
		fCutOff: Double;
		fTolerance: Double;
		fBlankWhenZero: Boolean;
		fBlankWhenNull: Boolean;
		fNullStr: string;
		function GetText: string; override;
		procedure SetForeColor (Value: TColor); override;
		function GetForeColor: TColor; override;
		procedure SetBackColor (Value: TColor); override;
		function GetBackColor: TColor; override;
		procedure DecideColor (const X: Double; var S: string;
			var FontColor: TColor);
		function GetVersion: string;
	public
		constructor Create (AOwner: TComponent); override;
		procedure Print (Report: TBaseReport); override;
	published
		property ColorLess: TColor
			read fLessColor
			write fLessColor
			default clred;
		property ColorEqual: TColor
			read fEqualColor
			write fEqualColor
			default clblack;
		property ColorGreater: TColor
			read fGreaterColor
			write fGreaterColor
			default clnavy;
		property CutOff: Double
			read fCutOff
			write fCutOff;
		property Tolerance: Double
			read fTolerance
			write fTolerance;
		property BlankWhenZero: Boolean
			read fBlankWhenZero
			write fBlankWhenZero
			default False;
		property BlankWhenNull: Boolean
			read fBlankWhenNull
			write fBlankWhenNull
			default False;
		property NullStr: string
			read fNullStr
			write fNullStr;
		property Version: string
			read GetVersion;
	end;

procedure RaveRegister;

implementation

uses
	RvDataField;

{$IFDEF DESIGNER}
{$R *.RES}
{$ENDIF}

procedure RaveRegister;

begin { RaveRegister }
	RegisterRaveGroup ('ESB', 'ESB Components');
	RegisterRaveComponents ('ESB', [TESBColNumDataText]);

	RegisterRaveProperties (TESBColNumDataText,
		{Beginner}'ColorLess;ColorEqual;ColorGreater;Cutoff;BlankWhenZero;BlankWhenNull;NullStr',
		{Intermediate}'Tolerance',
		{Developer}'',
		{Hidden}'Color');
	SetPropDesc (TESBColNumDataText, 'ColorLess',
		Trans ('The font color to use if the value is less than the Cutoff value.'));
	SetPropDesc (TESBColNumDataText, 'ColorEqual',
		Trans ('The font color to use if the value is equal to Cutoff value.'));
	SetPropDesc (TESBColNumDataText, 'ColorGreater',
		Trans ('The font color to use if the value is greater than the Cutoff value.'));
	SetPropDesc (TESBColNumDataText, 'CutOff',
		Trans ('Value to compare current value to decide whether to use ColorLess, ColorEqual or ColorGreater.'));
	SetPropDesc (TESBColNumDataText, 'BlankWhenZero',
		Trans ('When true if the value is zero, then just an "empty" field is shown.'));
	SetPropDesc (TESBColNumDataText, 'BlankWhenNull',
		Trans ('When true if the value is Null, then just an "empty" field is shown.'));
	SetPropDesc (TESBColNumDataText, 'NullStr',
		Trans ('When value is Null, this string is displayed, unless BlankWhenNull is true.'));
	SetPropDesc (TESBColNumDataText, 'Tolerance',
		Trans ('This enables you to "tweak" how accurately the CutOff and Zero comparisons are done.'));
end; { RaveRegister }

(*****************************************************************************}
( class TTeamNDColNumDataText
(*****************************************************************************)

constructor TESBColNumDataText.Create (AOwner: TComponent);
begin { Create }
	inherited Create (AOwner);
	fLessColor := clRed;
	fEqualColor := clBlack;
	fGreaterColor := clnavy;
	fCutOff := 0.0;
	fTolerance := 0.0001;
	fBlankWhenZero := False;
	fBlankWhenNull := False;
	fNullStr := '<Null>';
end; { Create }

function TESBColNumDataText.GetVersion: string;
begin
	Result := 'v1.1 of ESBTools';
end;

function TESBColNumDataText.GetText: string;
begin { GetText }
	Result := inherited GetText;
	//     if not ProjectManager.Printing then
	    //          Result := 'ColNum(' + Result + ')';
end; { GetText }

procedure TESBColNumDataText.SetForeColor (Value: TColor);
begin { SetForeColor }
	fGreaterColor := Value;
end; { SetForeColor }

function TESBColNumDataText.GetForeColor: TColor;
begin { GetForeColor }
     Result := fGreaterColor;
end; { GetForeColor }

procedure TESBColNumDataText.SetBackColor (Value: TColor);
begin { SetBackColor }
     fLessColor := Value;
end; { SetBackColor }

function TESBColNumDataText.GetBackColor: TColor;
begin { GetBackColor }
     Result := fLessColor;
end; { GetBackColor }

procedure TESBColNumDataText.DecideColor (const X: Double; var S: string;
     var FontColor: TColor);
begin
     if abs (X - fCutOff) < fTolerance then
     begin
          if fBlankWhenZero then
               S := ''
          else
               FontColor := fEqualColor
     end
     else if (X - fCutOff) < 0 then
          FontColor := fLessColor
     else
          FontColor := fGreaterColor;
end;

procedure TESBColNumDataText.Print (Report: TBaseReport);
var
     X: Double;
     S: string;
     FC: TColor;
begin { Print }
     with Report do
     begin
          S := Text;
          if Assigned (FontMirror) then
               AssignFont (FontMirror.Font)
          else
               AssignFont (Font);

          FC := Report.FontColor;
          try
               if Assigned (DataView) and (DataField <> '') then
               begin
                    if DataView.FieldByName (DataField).IsNull then
                    begin
                         if fBlankWhenNull then
                              S := ''
                         else
                              S := fNullStr;
                    end
                    else
                    begin
                         X := DataView.FieldByName (DataField).AsFloat;
                         DecideColor (X, S, FC);
                    end
               end
               else
               begin
                    X := StrToFloat (S);
                    DecideColor (X, S, FC);
               end
          except
               FC := Color;
          end;
          FontColor := FC;
          LineTop := PageTop;
          PrintJustify (S, PageLeft, FontJustify, 0.0, Width);
          FontColor := Color;
     end; { with }
end;

initialization
     RegisterProc ('ESBTools', RaveRegister);
end.

